@@ -8,6 +8,9 @@ use strict;
 use warnings;
 use vars qw(@ISA @EXPORT_OK @EXPORT %EXPORT_TAGS $VERSION);
 
+use constant DEBUG  => 0;
+use constant ASSERT => 1;
+
 require Exporter;
 
 @ISA = qw(Exporter);
@@ -20,7 +23,7 @@ our @export_default = qw();
 	       );
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 @EXPORT = ();
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 # Use XS for fast xors (TODO: make this optional)
 require XSLoader;
@@ -28,8 +31,6 @@ XSLoader::load('Net::OnlineCode', $VERSION);
 
 # on to our stuff ...
 
-use constant DEBUG => 0;
-
 
 # Codec parameters
 # q    is the number of message blocks that each auxiliary block will
@@ -64,8 +65,6 @@ use Fcntl;
 # * max degree variable (F)
 # * number of auxiliary blocks (0.55 *qen)
 # * probability distribution p_1, p2, ... , p_F
-#
-# All these are purely deterministic.
 
 sub new {
 
@@ -79,7 +78,7 @@ sub new {
 	      e           => 0.01,
 	      q           => 3,
 	      mblocks     => undef,
-	      expand_aux  => 1,
+	      expand_aux  => 0,
 	      e_warning   => 0,
 
 	      # We don't use or store any RNG parameter that's been
@@ -106,9 +105,6 @@ sub new {
   # does epsilon value need updating?
   my $f = _max_degree($e);
 
-  # try an alternative way of calculating F:
-  #  $f = $mblocks + $ablocks if $f > $mblocks + $ablocks;
-
   if ($f > $mblocks + $ablocks) {
 
     $e_changed = 1;
@@ -164,8 +160,6 @@ sub new {
 
   }
 
-  # how many auxiliary blocks would this scheme need?
-
   # calculate the probability distribution
   print "new: mblocks=$mblocks, ablocks=$ablocks, q=$q\n" if DEBUG;
   $P = _probability_distribution($mblocks + $ablocks,$e);
@@ -176,8 +170,11 @@ sub new {
 
   my $self = { q => $q, e => $e, f => $f, P => $P,
 	       mblocks => $mblocks, ablocks => $ablocks,
+	       coblocks => $mblocks + $ablocks,
                chblocks => 0, expand_aux=> $args{expand_aux},
-	       e_changed => $e_changed, unique => {} };
+	       e_changed => $e_changed, unique => {},
+	       fisher_string => pack("L*", (0 .. $mblocks + $ablocks -1)),
+	     };
 
   print "expand_aux => $self->{expand_aux}\n" if DEBUG;
 
@@ -246,9 +243,13 @@ sub _count_auxiliary {
   warn "failure probability " . ($delta ** $q) . "\n" if DEBUG;
   #$count = int(ceil($q * $delta * $n));
 
+  # Is it better to change q or the number of aux blocks if q is too
+  # big? It's certainly easier to keep the q value and increase the
+  # number of aux blocks, as I'm doing here, and may even be the right
+  # thing to do rather than ignoring the user's q value.
   if ($count < $q) {
-    #$count = $q;		# ???
-    #warn "updated _count_auxiliary output value to $q\n";
+    $count = $q;
+    warn "updated _count_auxiliary output value to $q\n";
   }
   return $count;
 }
@@ -386,7 +387,7 @@ sub _probability_distribution {
     # know there is a problem with the assumption!
     my $p_last = $sum + $pfterm / ($f * $f - $f);
     my $absdiff = abs (1 - $p_last);
-    warn "Absolute difference of 1,sum to p_F = $absdiff\n";
+    warn "Absolute difference of 1,sum to p_F = $absdiff\n" if $absdiff >1e-8;
   }
 
   return [@P,1];
@@ -402,34 +403,38 @@ sub _probability_distribution {
 # approaches the size of the array. That algorithm could make
 # exponentially many calls to rand, whereas this just makes one call
 # per item to be picked.
+#
+# 
 
 sub fisher_yates_shuffle {
 
   my ($rng, $array, $picks) = @_;
 
-  #print "picks is $picks\n";
-
-  die "fisher_yates_shuffle: 1st arg not an RNG object\n"
-    unless ref($rng);
-
-  die "fisher_yates_shuffle: 2nd arg not a listref\n"
-    unless ref($array) eq "ARRAY";
-
-  # Change recipe to pick subset of list
-  $picks=scalar(@$array) unless
-    defined($picks) and $picks >=0 and $picks<scalar(@$array);
-
-  # algorithm fills picks into the end of the array
-  my $i=scalar(@$array);
-  while (--$i >= scalar(@$array) - $picks) {
-    my $j=int($rng->rand($i + 1)); # range [0,$i]
-    #next if $i==$j;	           # not worth checking, probably
-    @$array[$i,$j]=@$array[$j,$i]
+  if (ASSERT) {
+    die "fisher_yates_shuffle: 1st arg not an RNG object\n"
+      unless ref($rng);
   }
 
-  # delete unpicked elements from the front of the array
-  # (does nothing if picks == length of the array)
-  splice @$array, 0, (scalar @$array - $picks);
+  # length in 32-bit words
+  my $len = length($array) >> 2;
+
+  # Change recipe to pick subset of list
+  $picks=$len unless
+    defined($picks) and $picks >= 0;
+
+  # algorithm fills picks into the end of the array
+  my $i=$len;
+  while (--$i >= $len - $picks) {
+    my $j=int($rng->rand($i + 1)); # range [0,$i]
+    #next if $i==$j;	           # not worth checking, probably
+    #    @$array[$i,$j]=@$array[$j,$i]
+    my $tmp1 = substr $array, $i << 2, 4;
+    my $tmp2 = substr $array, $j << 2, 4;
+    substr $array, $i << 2, 4, $tmp2;
+    substr $array, $j << 2, 4, $tmp1;
+  }
+
+  return (unpack "L*", substr $array, ($len - $picks) << 2);
 }
 
 #
@@ -482,19 +487,16 @@ sub auxiliary_mapping {
 
   my $aux_mapping = [];
 
+  my $ab_string = pack "L*", ($mblocks .. $mblocks + $ablocks -1);
+
   # list of empty hashes
   my @hashes;
   for (0 .. $mblocks + $ablocks -1) { $hashes[$_] = {}; }
 
   for my $msg (0 .. $mblocks - 1) {
     # list of all aux block indices
-    my $ab = [$mblocks .. $mblocks + $ablocks -1];
 
-    fisher_yates_shuffle($rng, $ab, $q);
-
-    #print "aux_mapping: shuffled list: " . (join " ", @$ab) . "\n";
-
-    foreach my $aux (@$ab) {
+    foreach my $aux (fisher_yates_shuffle($rng, $ab_string, $q)) {
       $hashes[$aux]->{$msg}=undef;
       $hashes[$msg]->{$aux}=undef;
     }
@@ -506,8 +508,6 @@ sub auxiliary_mapping {
     push @$aux_mapping, [ keys %{$hashes[$i]} ];
   }
 
-  #print "auxiliary_mapping: leaving RNG value: " . ($rng->as_hex). "\n";
-
   # save and return aux_mapping
   $self->{aux_mapping} = $aux_mapping;
 }
@@ -531,33 +531,17 @@ sub blklist_to_msglist {
       }
     } else {
       # aux block : push all message blocks it's composed of
-      push @xor_list, @{$self->{aux_mapping}->[$entry]};
+      my @expansion = @{$self->{aux_mapping}->[$entry]};
+      if (DEBUG) {
+	print "expand_aux: expanding $entry to " .
+	  (join " ", @expansion) . "\n";
+      }
+      push @xor_list, @expansion;
     }
   }
   return keys %blocks;
 }
 
-sub blklist_to_chklist  {
-
-  my $self = shift;
-  croak "This method only makes sense when called on a Decoder object!\n";
-}
-
-
-# non-method sub to toggle a key in a hash
-sub toggle_key {
-  my $href = shift;
-  my $key  = shift;
-
-  if (exists($href->{$key})) {
-    delete $href->{$key};
-  } else {
-    # apparently, using key => undef is more space-efficient than
-    # using key => 1 (similar changes made throughout this file)
-    $href->{$key}=undef;
-  }
-}
-
 # Calculate the composition of a single check block based on the
 # supplied RNG. Returns a reference to a list of composite blocks
 # indices.
@@ -569,93 +553,25 @@ sub checkblock_mapping {
 
   croak "rng is not an object reference\n" unless ref($rng);
 
-  my $coblocks = $self->get_coblocks;
-  my $P        = $self->{P};
+  my ($mblocks,$coblocks,$P) = @{$self}{"mblocks","coblocks","P"};
+  my @coblocks;
 
-  #  die "Probability distribution has wrong number of terms\n"
-  #    unless scalar(@$P) <= $coblocks;
+  # use weighted distribution to find how many blocks to link
+  my $i = 0;
+  my $r = $rng->rand;
+  ++$i while($r > $P->[$i]);	# terminates since r < P[last]
+  ++$i;
 
-  my $check_mapping;
+  # select i composite blocks uniformly
+  @coblocks = fisher_yates_shuffle($rng, $self->{fisher_string} , $i);
 
-  # It's possible to generate a check block that is empty. If it only
-  # includes message blocks, then there's no problem. However, if the
-  # expansion of all the auxiliary blocks is equal to the list of
-  # message blocks then two two cancel out. Besides being inefficient
-  # to transmit effectively empty check blocks, it can also cause a
-  # bug in the decoder where it assumes that the expanded list of
-  # blocks is not empty. The solution is the same for both encoder and
-  # decoder (loop until expansion is not empty), so I'm implementing
-  # it here in the base class.
-  #
-  # Note that although this involves expanding auxiliary blocks, for
-  # the moment, I'm ignoring "expand_aux" option and will just return
-  # the unexpanded list. This may change in future once I've had a
-  # chance to look at the problem more closely.
-
-  my $mblocks = $self->{mblocks}; # quicker than calling is_message
-  my %expanded=();
-  my $tries = 0;
-  my $key;			# used for uniqueness-checking
-  until (keys %expanded) {
-
-    ++$tries;
-
-    # use weighted distribution to find how many blocks to link
-    my $i = 0;
-    my $r = $rng->rand;
-    ++$i while($r > $P->[$i]);	# terminates since r < P[last]
-    ++$i;
-
-    #die "went past end of probability list\n" if $i > @$P;
-
-    #warn "picked $i values for checkblock (from $coblocks)\n";
-
-    # select i composite blocks uniformly
-    $check_mapping = [ (0 .. $coblocks-1) ];
-    fisher_yates_shuffle($rng, $check_mapping, $i);
-
-    # check block for uniqueness before expansion
-    $key = join " ", sort { $a <=> $b } @$check_mapping;
-    if (exists $self->{unique}->{$key}) {
-      warn "quashed duplicate check block\n" if DEBUG;
-      next;
-    }
-
-    # print "check_mapping: raw composite block list: ", 
-    #  (join " ", @$check_mapping), "\n";
-
-    # check expanded list
-    my @xor_list = @$check_mapping;
-    while (@xor_list) {
-      my $entry = shift @xor_list;
-      if ($entry < $mblocks) { # is it a message block index?
-	# toggle entry
-	toggle_key (\%expanded, $entry);
-      } else {
-	# aux block : push all message blocks it's composed of. Since
-	# we're sharing the aux_mapping structure with the decoder, we
-	# have to filter out any entries it's putting in (ie,
-	# composite blocks) or we can get into an infinite loop
-	my @expanded = grep { $_ < $mblocks } @{$self->{aux_mapping}->[$entry]};
-	#print "check_mapping: expanding aux block $entry to ", 
-	#  (join " ", @expanded), "\n";
-	push @xor_list, @expanded;
-      }
-    }
+  if (ASSERT) {
+    die "fisher_yates_shuffle: created empty check block\n!" unless @coblocks;
   }
 
-  # prevent generating this block again
-  $self->{unique}->{$key}=undef;
+  print "CHECKblock mapping: " . (join " ", @coblocks) . "\n" if DEBUG;
 
-  #warn "Created unique, non-empty checkblock on try $tries\n" if $tries>1;
-
-  die "fisher_yates_shuffle: created empty check block\n!" unless @$check_mapping;
-
-  ++($self->{chblocks});
-
-  print "CHECKblock mapping: " . (join " ", @$check_mapping) . "\n" if DEBUG;
-
-  return $check_mapping;
+  return \@coblocks;
 
 }
 
@@ -708,9 +624,26 @@ Net::OnlineCode - A rateless forward error correction scheme
 
   use strict;
 
-  # Base class only exports routines for doing xor
+  # Base class can export routines for doing xor
   use Net::OnlineCode ':xor';
 
+  # Use the constructor to examine what algorithm parameters
+  # would be generated for a given number of message blocks
+
+  my $o = Net::OnlineCode->new(
+    mblocks => 100_000,  # required parameter
+    e => 0.01, q => 3    # optional/default values
+  );
+
+  my $e  = $o->get_e;    # The 'e' and 'q' values may be
+  my $q  = $o->get_q;    # updated for some values of mblocks
+  my $f  = $o->get_f;    # calculated from 'mblocks', 'e' and 'q'
+
+  # At this point, you would normally destroy this object and
+  # re-create an Encoder and/or Decoder object using the same
+  # parameters as extracted above.
+
+  # some strings to demonstrate use of *_xor_* routines
   my @strings = ("abcde", "     ", "ABCDE", "\0\0\0\0\0");
 
   # xor routines take a reference to a destination string, which is
@@ -725,14 +658,45 @@ Net::OnlineCode - A rateless forward error correction scheme
 
 =head1 DESCRIPTION
 
-This module implements the common functions required for the
-L<Net::OnlineCode::Encoder> and L<Net::OnlineCode::Decoder> modules.
-Apart from the two xor library routines shown above, there are no
-other user-callable methods or functions.
+This section will not make much sense to you unless you know what Online Codes are and how they work. If this is your first time reading this page, I advise you to skip ahead to the next section.
 
-The remainder of this document will give a brief overview of the
-Online Code algorithms. For a programmer's view of how to use this
-collection of modules, refer to the man pages for the
+This module provides a base class for the
+L<Net::OnlineCode::Encoder> and L<Net::OnlineCode::Decoder> modules.
+It can also be used directly in user code:
+
+=over
+
+=item * to find out what parameters are generated for a given number of message blocks
+
+=item * to access the exported XOR routines
+
+=back
+
+The constructor takes a required 'mblocks' parameter and, optionally,
+values for 'e' and 'q' (described below). It then checks whether this
+combination of parameters makes sense. If it doesn't, it will change
+the 'e' or 'q' values to something more appropriate.
+
+There is usually no need to call the constructor directly since it
+will be called during the creation of Net::OnlineCode::Encoder or
+Net::OnlineCode::Decoder objects. However, if you wish to examine what
+changes were made to the supplied parameters, it is more efficient to
+call the base class's constructor directly since it avoids some costly
+initialisation code included in the Encoder/Decoder classes.
+
+The remaining two exported functions can be used to XOR pairs of
+strings. Since the Online Code scheme is based on XORing blocks of
+data, the base class provides two implementations to assist with
+this. One implementation (safe_xor_strings) is a slow, pure Perl sub,
+while the other (fast_xor_strings) uses a call to a faster C
+routine. This module (and the derived Encoder and Decoder module) only
+deal with the details necessary to implement the Online Code
+algorithms, but do not store or XOR blocks automatically: those tasks
+are left for you to implement in whatever way you want.
+
+The remainder of this document will give a brief overview of what
+Online Codes are and how they work. For a programmer's view of how to
+use this collection of modules, refer to the man pages for the
 L<Encoder|Net::OnlineCode::Encoder> and
 L<Decoder|Net::OnlineCode::Decoder> modules.
 
@@ -757,9 +721,10 @@ schemes in two important respects:
 
 When using a traditional error-correction scheme, the sender usually
 has to set up the encoder parameters to take account of the expected
-packet loss rate. In contrast, with Online Codes, the sender
-effectively doesn't care about what the packet loss rate: it just
-keeps sending new check blocks until either the receiver(s)
+packet loss rate, and if the loss rate is greater than this, the file
+generally cannot be recovered at all. In contrast, with Online Codes,
+the sender effectively doesn't care about what the packet loss rate:
+it just keeps sending new check blocks until either the receiver(s)
 acknowledge the message as having been decoded, or until it has a
 reasonable expectation that the message should have been decoded.
 
@@ -810,15 +775,15 @@ composite blocks comprise each check block.
 
 Naturally, for this to work, not only should the sender and receiver
 both be using the same PRNG algorithm, but they also need to be using
-the same PRNG I<seeds>. This is where I<Block IDs> (and also, for the
-auxiliary block mapping, I<File IDs>) come in. For check blocks, the
-sender picks a (truly) random Block ID and uses it to seed the
-PRNG. Then, using the PRNG, it pseudo-randomly selects some number of
-composite blocks. It then sends the Block ID along with the xor of all
-the selected composite blocks. The sender then uses the Block ID to
-seed its own PRNG, so when it pseudo-randomly selects the list of
-composite blocks, it should be the same as that selected by the
-sender.
+the same PRNG I<seeds>. This is where I<Block IDs> (and also, during
+the Encoder/Decoder construction phase , I<File IDs>) come in. For
+check blocks, the sender picks a (truly) random Block ID and uses it
+to seed the PRNG. Then, using the PRNG, it pseudo-randomly selects
+some number of composite blocks. It then sends the Block ID along with
+the xor of all the selected composite blocks. The sender then uses the
+Block ID to seed its own PRNG, so when it pseudo-randomly selects the
+list of composite blocks, it should be the same as that selected by
+the sender.
 
 A similar scheme is used at the start of the transmission to determine
 which message blocks are xored to create the auxiliary blocks.
@@ -839,11 +804,9 @@ variations:
 
 =item * in the case where the epsilon value would produce a max degree (F) value greater than the number of message blocks, it (ie, epsilon) is increased until F <= # of message blocks. The code to do this is based on Python code by Gwylim Ashley.
 
-=item * duplicate check blocks are quashed (since they provide no new information)
+=item * the graph decoding algorithm also includes an "auxiliary rule", which allows decoding (solving) an auxliary block that comprises only solved message blocks
 
-=item * the graph decoding algorithm is replaced with a functionally equivalent one
-
-=item * message blocks need not decoded immediately: rather, the calling application makes the choice about when to do the xors (this I<may> be more time-efficient, particularly if the application is storing check blocks to secondary storage)
+=item * message blocks are decoded immediately: rather, the calling application makes the choice about when to do the xors (this I<may> be more time-efficient, particularly if the application is storing check blocks to secondary storage, but in any event, decoupling the Online Code algorithm from the XOR step allows much more flexibility)
 
 =back
 
@@ -852,25 +815,18 @@ This module implements one using SHA-1, since it is portable across
 platforms and readily available.
 
 
-=head1 IMPORTANT NOTE
+=head1 RELEASE NOTE FOR V0.03
 
-This is the initial software release. It should be free of serious
-bugs, however it has a few shortcomings:
+This version of the code is a major advance from the previous
+version. Most of the algorithms and API are the same, but this version
+is much more optimised, both in terms of speed and memory usage. The
+various changes that have been implemented are documented in the
+RELEASES file at the top level of distribution tarball.
 
-=over
-
-=item * Memory usage is particularly high, especially if the number of check blocks runs into the thousands; and
-
-=item * The Decoder must see more check blocks than the original paper suggests should be the case
-
-=item * Although intended to be used with POE, the Decoder is currently not very suitable for use in a POE callback because it does not yield until it has done as much graph decoding as possible. If POE is being used to handle incoming network packets, the delay can cause it to drop packets. A future version will correct this by reducing the amount of work done per callback (so that control is returned to the POE event loop sooner, with the possibility to queue the remaining outstanding work with more calls to the Decoder object)
-
-=back
-
-As a result, it is likely that any future version that tries to
-address these two points may result in changes to the calling
-interfaces. See the TODO file that came in this distribution for more
-details.
+I consider this version to be stable. While performance has been
+improved considerably, it may still be too slow for some applications.
+As a result, I intend to re-implement critical parts of it in C for
+the next release.
 
 =head1 SEE ALSO
 
@@ -888,18 +844,39 @@ PDF/PostScript links:
 
 L<Github repository for Gwylim Ashley's Online Code implementation|https://github.com/gwylim/online-code> (various parts of my code are based on this)
 
+There are various test/demo programs in the tests directory in this
+distribution that can serve as a reference for how to use these
+modules. In particular, the following can be useful examples to base
+your own code on:
+
+=over
+
+=item * I<mindecoder.pl> is a minimal decoder that doesn't do any XORing of data
+
+=item * I<codec.pl> does both encoding and decoding of a real message string
+
+=item * I<probdist.pl> shows the effects that various message block counts have on the 'e' and 'q' parameters
+
+=back
+
+
 This module is part of the GnetRAID project. For project development
 page, see:
 
   https://sourceforge.net/projects/gnetraid/develop
 
+I am also moving over to using github for all future development:
+
+  https://github.com/declanmalone/gnetraid
+
+
 =head1 AUTHOR
 
 Declan Malone, E<lt>idablack@users.sourceforge.netE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013 by Declan Malone
+Copyright (C) 2013-2015 by Declan Malone
 
 This package is free software; you can redistribute it and/or modify
 it under the terms of the "GNU General Public License" ("GPL").
